package org.gcube.gcat;

import java.io.IOException;

import org.gcube.common.authorization.utils.manager.SecretManagerProvider;
import org.gcube.gcat.configuration.isproxies.FacetBasedISConfigurationProxy;
import org.gcube.gcat.configuration.isproxies.FacetBasedISConfigurationProxyFactory;
import org.gcube.gcat.rest.RequestFilter;
import org.gcube.smartgears.ApplicationManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class GCatInitializator implements ApplicationManager {

	/**
	 * Logger
	 */
	private static Logger logger = LoggerFactory.getLogger(GCatInitializator.class);
	
	/** 
	 * {@inheritDoc}
	 * The method discover the plugins available on classpath and their own 
	 * supported capabilities and publish a ServiceEndpoint with the 
	 * discovered information.
	 * Furthermore create/connect to DB
	 */
	@Override
	public void onInit() {
		RequestFilter requestFilter = new RequestFilter();
		try {
			requestFilter.filter(null);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		String context = SecretManagerProvider.instance.get().getContext();
		
		logger.trace(
				"\n-------------------------------------------------------\n"
				+ "GCat is Starting on context {}\n"
				+ "-------------------------------------------------------",
				context);
		
		FacetBasedISConfigurationProxy facetBasedISConfigurationProxy = FacetBasedISConfigurationProxyFactory.getInstance(context);
		try {
			facetBasedISConfigurationProxy.createCallsForToVirtualService();
		}catch (Exception e) {
			logger.warn("Gcat is not configured through the Facet Based IS in context {}. Please create/addToContext the expected resources ASAP. The Gcore IS will be used.", context, e);
		}
		
		logger.trace(
				"\n-------------------------------------------------------\n"
				+ "GCat Started Successfully on context {}\n"
				+ "-------------------------------------------------------",
				context);
		
	}
	
	/** 
	 * {@inheritDoc} 
	 * This function is invoked before the service will stop and unpublish the 
	 * resource from the IS to maintain the infrastructure integrity.
	 * Furthermore close the connection to DB.
	 */
	@Override
	public void onShutdown(){
		RequestFilter requestFilter = new RequestFilter();
		try {
			requestFilter.filter(null);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		String context = SecretManagerProvider.instance.get().getContext();
		
		logger.trace(
				"\n-------------------------------------------------------\n"
				+ "GCat is Stopping on context {}\n"
				+ "-------------------------------------------------------", 
				context);
		
		/*
		FacetBasedISConfigurationProxy facetBasedISConfigurationProxy = FacetBasedISConfigurationProxyFactory.getInstance(context);
		try {
			facetBasedISConfigurationProxy.deleteCallsForToVirtualService();
		}catch (Exception e) {
			logger.warn("Gcat is not configured through the Facet Based IS in context {}. Please create/addToContext the expected resources ASAP. The Gcore IS will be used.", context, e);
		}
		*/
		
		logger.trace(
				"\n-------------------------------------------------------\n"
				+ "GCat Stopped Successfully on context {}\n"
				+ "-------------------------------------------------------", 
				context);
	}
}
